########################################################################
# Introscope Kstat Perl Library
#                                                                      
# CA Wily Introscope(R) Version 9.7.1 Release 9.7.1.16
# Copyright (c) 2015 CA. All Rights Reserved.
# Introscope(R) is a registered trademark of CA.
#
########################################################################
# Kstat Module
#-------------
# This module abstracts the interface to Solaris Kernel statistics.
# If the Sun::Solaris::Kstat module exists, it is used, otherwise
# the Solaris command-line utility, kstat, is used. The former (perl lib)
# is preferred due to performance reasons, but access to the latter (cmd-line)
# is provided in case the Sun kstat perl library is not installed or 
# cannot be found (e.g. if using another version of Perl). If the 
# command-line utility does not exist, the plugin exits with an error 
# message.
# 
########################################################################

package Wily::Kstat;

use strict;

##########
# GLOBALS
##########
my $KSTAT_CMD  = '/usr/bin/kstat -p';
my $perl_module_exists;
my $use_regex = 1;

##############
# Subroutines
##############

sub useRE {
    $use_regex = shift;
}

sub containsOnlyShellWildCards {
    # only detects very simple wild-cards ([\w*?-:.])
    #
    my @args = @_;
    my $flag = 1;
    foreach my $arg (@args) {
        if ($arg !~ /^[\w\-:\.\*\?]+$/) {
            $flag = 0;
            last;
        }
    }
    return $flag;
}

sub convertShellWildCardsToRE {
    my @src = @_;
    my @dst = ();

    foreach my $arg (@src) {
        $arg =~ s/\*/\.\*/g;
        $arg =~ s/\?/\./g;
        my $argRE = qr($arg);
        push(@dst, $argRE);
    }

    return @dst;
}

sub filterRE {
    # If any of the arg have form: /<regex>/ 
    # assume it's a regex and strip the surrounding slashes
    #
    my @src = @_;
    my @dst = ();

    foreach my $arg (@src) {
        $arg =~ s@^/(.*)/$@$1@;
        if ($use_regex) {
            $arg = '.' if !$arg;
        } 
        push(@dst, $arg);
    }
    return @dst;
}

# This routine either uses the Sun::Solaris::Kstat module if it exists
# or the kstat cmd-line if not
#
sub getStats {
    my %args     = @_;
    my $classRE  = $args{class};
    my $moduleRE = $args{module};
    my $instRE   = $args{instance};
    my $nameRE   = $args{name};
    my $statRE   = $args{statistic};

    my $kstatRef; 
    my %kstats;
    my %names;

    my $only_shell_wildcards = 
        &containsOnlyShellWildCards($classRE, $moduleRE, $instRE, $nameRE, $statRE);

    if ($perl_module_exists) {
        #----------------------------------------------------
        # Perl module (Sun::Solaris::Kstat) exists, so use it
        #----------------------------------------------------
        $kstatRef = Sun::Solaris::Kstat->new();

        if (!$use_regex && $only_shell_wildcards) {
            ($classRE, $moduleRE, $instRE, $nameRE, $statRE) = 
                &convertShellWildCardsToRE($classRE, $moduleRE, $instRE, $nameRE, $statRE);
        }

        my @moduleList = sort keys %{$kstatRef};
        @moduleList = grep(/$moduleRE/, @moduleList);
        foreach my $aModule (@moduleList) {
            my @instList = sort keys %{$kstatRef->{$aModule}}; 
            @instList = grep(/$instRE/, @instList);
            foreach my $aInstance (@instList) {
                my @nameList = sort keys %{$kstatRef->{$aModule}{$aInstance}}; 
                @nameList = grep(/$nameRE/, @nameList);
                foreach my $aName (@nameList) {
                    if ( ! $names{$aName} ) {  # Don't process duplicate names.
                        $names{$aName} = 1;    # Mark this name as processed.
	                     my @statList = sort keys %{$kstatRef->{$aModule}{$aInstance}{$aName}};
    	                  @statList = grep(/$statRE/, @statList);
        		            foreach my $aStat (@statList) {
            	             my $aClass = $kstatRef->{$aModule}{$aInstance}{$aName}{class};
                	          if ($aClass =~ /$classRE/) {
                    	           my $value = $kstatRef->{$aModule}{$aInstance}{$aName}{$aStat};
                     	        # $value = s/\000//g; # nulls can get in there
                                ### print "### ($aClass)$aModule:$aInstance:$aName:$aStat = <" .  $value . ">\n" if $value =~ /sparc/;
                                $kstats{$aModule}{$aInstance}{$aName}{$aStat} = $value;
                                $kstats{$aModule}{$aInstance}{$aName}{class} = $aClass;
                            } # if ($aClass =~ /$classRE/)
                        } # foreach my $aStat (@statList)
                    } # if ( ! $names{$aName} )
                } # foreach my $aName (@nameList) 
            } # foreach my $aInstance (@instList)
        } # foreach my $aModule (@moduleList)

    } else {
        #------------------------------------------------------
        # No perl module (Sun::Solaris::Kstat), so use cmd-line
        #------------------------------------------------------
        my ($aClass, $aInstance, $aModule, $aName, $aStatistic, $aValue);

        my $cmd = "$KSTAT_CMD"; 

        # If no -regex (i.e. no force use of regex)
        # and only shell wildcards, we want to pre-filter
        # with kstat cmd-line (using -c, -m, etc.), so
        # no need to filter at this level.
        # Otherwise, we skip this block and get everything,
        # results being parsed later inside the loop.
        #
        if (!$use_regex && $only_shell_wildcards) {
            $cmd .= " -c $classRE"  if ($classRE  ne '.');
            $cmd .= " -m $moduleRE" if ($moduleRE ne '.');
            $cmd .= " -i $instRE"   if ($instRE   ne '.');
            $cmd .= " -n $nameRE"   if ($nameRE   ne '.');
            $cmd .= " -s $statRE"   if ($statRE   ne '.');
        }

        open(KSTAT, "$cmd |") || die "kstat cmd-line <$cmd> could not be found";
        while (my $line = <KSTAT>) {
            chomp $line;
            next if ($line =~ /^\s*$/); # skip blank lines

            if ($line =~ /^([^:]+):([^:]+):([^:]+):(\S+)\s+([\+\-\w.,]+)/) {
                $aModule    = $1;
                $aInstance  = $2;
                $aName      = $3;
                $aStatistic = $4;
                $aValue     = $5;
                if ( ! $names{$aName} ) {  # Don't process duplicate names.
                    $names{$aName} = 1;    # Mark this name as processed.
	                 if ( $use_regex || !$only_shell_wildcards) {
	                     if ( ($aModule     =~ $moduleRE) &&
	                          ($aInstance   =~ $instRE)   &&
	                          ($aName       =~ $nameRE)   &&
	                          ($aStatistic  =~ $statRE)      ) {
	                         $kstats{$aModule}{$aInstance}{$aName}{$aStatistic} = $aValue;
    	                  }
	                 } else {
	                     # We get here if no -regex (i.e. no force to use regex)
	                     # and only shell wildcards. Means we've pre-filtered
	                     # with kstat cmd-line (using -c, -m, etc.), so
	                     # no need to filter at this level.
	                     #
	                     $kstats{$aModule}{$aInstance}{$aName}{$aStatistic} = $aValue;
    	              }
    	          } # if ( ! $names{$aName} )
            } else {                                
                # ERROR: unexpected line format
                print STDERR "### WARNING: Bad line format parsing '$cmd' output: <$line>\n";
                print STDERR "### WARNING: Could be due to bugs in 'kstat' cmd-line\n"
                #
                # Swallow for now...some bugs in kstat cmd-line output
                # which causes "unix:0:segmap:  ", "hme:0:hme0:  "
            }
        }
        close(KSTAT);
    }
    $kstatRef = \%kstats;
    return $kstatRef;
}

sub printStats {
    my %args = @_;
    my $kstat         = $args{stats};
    my $printInstance = $args{printInstance};

    my @moduleList = sort keys %{$kstat};
    foreach my $aModule (@moduleList) {
        my @instList = (sort keys %{$kstat->{$aModule}});
        foreach my $aInstance (@instList) {
            my @nameList = sort keys %{$kstat->{$aModule}{$aInstance}}; 
            foreach my $aName (@nameList) {
                my @statList = sort keys %{$kstat->{$aModule}{$aInstance}{$aName}};
                foreach my $aStat (@statList) {
                    my $value = $kstat->{$aModule}{$aInstance}{$aName}{$aStat};
                    my $aClass = $kstat->{$aModule}{$aInstance}{$aName}{class};
					### print "### ($aClass)$aModule:$aInstance:$aName:$aStat = <" .  $value . ">\n" if $value =~ /sparc/;
                    my $subresource;
                    if ($printInstance) {
                        $subresource = "$aModule|$aInstance|$aName";
                    } else {
                        $subresource = "$aModule|$aName";
                    }
                    if ($aStat ne 'class') {
			### may contain null, need to remove null
			$aClass =~ s/\000//g;
			$subresource =~ s/\000//g;
			$value =~ s/\000//g;
			$aStat =~ s/\000//g;

			if ($value =~ /^[0-9]+\.[0-9]+[e|E][+|-][0-9]+$/ || $value =~ /^[0-9]+\.[0-9]+/) {
			    ### This is a float, so we round it up
			    $value = int($value + .5 * ($value <=> 0));
			}
			if ($value =~ /^[0-9]+$/) {
			    if ($value > 2143487648) {
				### This is too big for a long to hold
				### so we'll treat it as a string
                            	Wily::PrintMetric::printMetric( 
                                    type => 'StringEvent',
                                    resource => $aClass,
                                    subresource => $subresource,
                                    name => $aStat,
                                    value => $value,
                                  );
			    }
			    else {
                            	Wily::PrintMetric::printMetric( 
                                    type => 'LongCounter',
                                    resource => $aClass,
                                    subresource => $subresource,
                                    name => $aStat,
                                    value => $value,
                                  );
			    }
			}
			else  {
			    ### This is a string
				if($value ne ''){
                            Wily::PrintMetric::printMetric( 
                                    type => 'StringEvent',
                                    resource => $aClass,
                                    subresource => $subresource,
                                    name => $aStat,
                                    value => $value,
                                  );
				}
			}
		    }
                } # foreach my $aStat (@statList)
            } # foreach my $aName (@nameList) 
        } # foreach my $aInstance (@instList) 
    } # foreach my $aModule (@moduleList)
}

#######
# MAIN
#######

# This logic simply checks if the Solaris kstat perl module
# exists (eval stmt), and sets a file global flag ($perl_module_exists).
#

eval {
    require Sun::Solaris::Kstat;
};

if ($@) { # if $@ is defined, it means the eval failed with an error
    $perl_module_exists = 0;
} else { 
    $perl_module_exists = 1;
}

1;
