########################################################################
# Introscope EPAgent Plugin Script                               
#                                                                      
# CA Wily Introscope(R) Version 9.7.1 Release 9.7.1.16
# Copyright (c) 2015 CA. All Rights Reserved.
# Introscope(R) is a registered trademark of CA.
#
########################################################################
# diskStats.pl 
# ------------
# This Plugin Script returns metrics containing information
# about the system harddisks.
#
# diskStats.pl takes disk names as command line arguments.
# If no disk names are specified, the information about all
# disks is reported.
#
# Example: perl diskStats.pl
#            -- This reports metrics on all local disks
# Example: perl diskStats.pl / /usr
#            -- This reports metrics on only the disks '/' and '/usr'
#
########################################################################

#import our modules
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;

# get the mounted disks specified on the command line
my $mountedDisksRegEx = '.'; # default is match all
if (scalar(@ARGV) > 0) {
    $mountedDisksRegEx = join('|', @ARGV); 
}

# Build the iostat command with the proper arguments
my $iostatCommand = 'iostat -xnPr';
my $dfCommand = 'df -k';



# Get the disk stats
my @iostatResults = `$iostatCommand`;
my @dfResults = `$dfCommand`;
# Get rid of the header lines for each command
@iostatResults = @iostatResults[2..$#iostatResults];
@dfResults = @dfResults[1..$#dfResults];

# Compile all the disk names reported in df
my %dfDisks = (); # hash with key = device name, 
                  # value = the line returned by df for this device
foreach my $j (@dfResults) {
  # get rid of any trailing \n char
  chomp $j;

  my @line = split(/\s+/, $j);
  # the first item is the filesystem name
  # add the disk name as the key
  # and the stats reported by df as the value
  $dfDisks{$line[0]} = $j;
}

# The in-order column information
# (excluding device name) returned by
# iostat.  The column headers are the keys and the
# metric types are the values.
my %ioStatHeaderInfo = ('Reads/sec'  => 'IntCounter',
            'Writes/sec' => 'IntCounter',
            'KB Read/sec'=> 'IntCounter',
            'KB Written/sec' => 'IntCounter',
            'Avg. Transactions Waiting' => 'IntCounter', 
            'Avg. Transactions Active'  => 'IntCounter',
            'Avg. Service Time in Wait Queue (Milliseconds)' 
            => 'IntCounter',
            'Avg. Service Time Active Transactions (Milliseconds)'
            => 'IntCounter',
            '% Time Transactions Waiting' => 'IntCounter', 
            '% Time Disk Is Busy' => 'IntCounter');


# parse the iostat results and report the
# relevant data using metrics
foreach my $line (@iostatResults) {

  # if there's a trailing \n char, get rid of it
  chomp $line;
  
  # the -r option for iostat prints out the stats separated by
  # commas, so grab all of them
  my @deviceStats = split(',', $line);
  my $device = $deviceStats[10];
  # see if the disk was reported by df
  # and report the io and df stats if it was
  foreach my $disk (keys %dfDisks) {
    if($disk =~ /$device/) {
      my $dfLine = $dfDisks{$disk};
      my @dfStats = split /\s+/, $dfLine;
      my $diskName = $dfStats[5];

      # now, check to see if the user specified this disk on the command
      # line.
      next if $diskName !~ /$mountedDisksRegEx/i;

      # report the df stats
      # Just print the Free Disk Space as a Percent and in Megabytes
      # chop gets rid of '%' in the capacity
      chop $dfStats[4];
      ### Wily::PrintMetric::printMetric('IntCounter',
      ###    "Disk|$diskName:Free Disk Space (%)",
      ###    100 - $dfStats[4]); 

      Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                      resource    => 'Disk',
                                      subresource => $diskName,
                                      name        => 'Free Disk Space (%)',
                                      value       => 100 - $dfStats[4],
                                    );

                                     
      ### Wily::PrintMetric::printMetric('IntCounter',
      ###                    "Disk|$diskName:Free Disk Space (MB)",
      ###                    int ($dfStats[3] / 1024));

      Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                      resource    => 'Disk',
                                      subresource => $diskName,
                                      name        => 'Free Disk Space (MB)',
                                      value       => int ($dfStats[3] / 1024),
                                    );

      # report the iostats
      for(my $j = 0; $j <= 9; $j++) {
        my @headers = (keys %ioStatHeaderInfo);
        my $metric = $headers[$j];

        # Report the metric

        ### Wily::PrintMetric::printMetric("$ioStatHeaderInfo{$metric}", 
        ###                "Disk|$diskName:$metric",
        ###                int ($deviceStats[$j]));

        Wily::PrintMetric::printMetric( type     => $ioStatHeaderInfo{$metric},
                                        resource    => 'Disk',
                                        subresource => $diskName,
                                        name        => $metric,
                                        value       => int ($deviceStats[$j]),
                                      );
      }
    }
  }
}


