########################################################################
# Introscope EPAgent Plugin Script                               
#                                                                      
# CA Wily Introscope(R) Version 9.7.1 Release 9.7.1.16
# Copyright (c) 2015 CA. All Rights Reserved.
# Introscope(R) is a registered trademark of CA.
#
########################################################################
# processAvailability.pl
# ----------------------
# This Plugin Script returns a metric with a value equal to
# the number of process names that match the passed in argument value.
# processAvailability.pl takes 1 required command line option
# and 4 optional command line options:
#    1) match  - the Perl regular expression to search for.
#    2) resource  - the resource part of the metric name
#    3) subResource - the sub-resource part of the metric name
#    4) metric - the main part of the metric name
#    5) distinctMatch - this has no value and determines whether to 
#                       keep and report counts for each regular expression 
#                       match separately. This option overrides the value of
#                       the subResource argument if it is specified.
#
# The type of metric returned is 'IntCounter'.
#
# Example: perl processAvailability.pl -match ^sh
#            -- this reports the number of processes that begin with the
#            -- characters 'sh'
#
########################################################################

#import our modules
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;
use Win32::PerfLib;
use Getopt::Long;

# Metric Name Constants
my $RESOURCE  = 'RunningProcesses';
my $SUBRESOURCE = ''; # this is provided as the reg exp to search for or
                 # specified on the command line
my $METRIC = 'Count';

my $METRICTYPE = 'IntCounter';

my $match  = ''; # the pattern to search for
my $distinctMatch = ''; #do we want distinct matches to be counted separately?

# Get and Check command line arguments here...
GetOptions ('match=s' => \$match,
        'resource=s' => \$RESOURCE,
        'subResource=s' => \$SUBRESOURCE,
        'metric=s' => \$METRIC,
        'distinctMatch' => \$distinctMatch
       );

if($match eq '') {
  my $errStr = 'usage \'perl processAvailability.pl -match <Regular Expression Process Name> '; 
  $errStr .= '[-resource <Resource Name>] [-subResource <Sub-Resource Name>] [-metric <Metric Name>] ';
  $errStr .= '[-distinctMatch]';
  die "$errStr";
}

# if there was no subResource specified,
# then make it the match value
if($SUBRESOURCE eq '') {
  $SUBRESOURCE = $match;
}

# Get the process data using PerfLib
my $server;
my %counter;
Win32::PerfLib::GetCounterNames($server, \%counter);
my %r_counter = map { $counter{$_} => $_ } keys %counter;
my $process_obj = $r_counter{Process};
my $process_id = $r_counter{'ID Process'};
my $perflib = new Win32::PerfLib($server);
my $proc_ref = {};
$perflib->GetObjectList($process_obj, $proc_ref);
$perflib->Close();

# Use PerfLib to get all the process names
my $instance_ref = $proc_ref->{Objects}->{$process_obj}->{Instances};
my %process;
foreach my $p (keys %{$instance_ref}) {
  my $counter_ref = $instance_ref->{$p}->{Counters};
  foreach my $i (keys %{$counter_ref}) {
    next if $instance_ref->{$p}->{Name} eq "_Total";
    # only grab the process names
    if($counter_ref->{$i}->{CounterNameTitleIndex} == $process_id) {
      $process{$counter_ref->{$i}->{Counter}} =    $instance_ref->{$p}->{Name};
    }
  }
}
# Count the number of matches
my $count = 0;
my %distinctCount;
foreach my $p (keys %process) {
  if($process{$p} =~ /$match/i) {
    if($distinctMatch ne '') {
      $distinctCount{$process{$p}}++;
    } else {
      $count++;
    }
  }
}

# print the metrics properly
if($distinctMatch ne '') {
  foreach my $i (keys %distinctCount) {
    # Report each metric
    ### Wily::PrintMetric::printMetric($METRICTYPE, "$RESOURCE|$i:$METRIC", $distinctCount{$i});
    &Wily::PrintMetric::printMetric( type => $METRICTYPE, 
                                     resource => $RESOURCE,
                                     subresource => $i,
                                     name => $METRIC, 
                                     value => $distinctCount{$i}
                                   );
  }
} else {
  # Report the metric
  ### Wily::PrintMetric::printMetric($METRICTYPE, "$RESOURCE|$SUBRESOURCE:$METRIC", $count);
  &Wily::PrintMetric::printMetric( type => $METRICTYPE, 
                                   resource => $RESOURCE,
                                   subresource => $SUBRESOURCE,
                                   name => $METRIC, 
                                   value => $count
                                 );
}


